             .title        "VRC6 Reader Plugin"


;10/29/00 
;Written by KH
;Version 1.0

;07/26/07
;Bugfix by BootGod - changed to mapper controlled mirroring

;03/16/09
;Added user var support for both VRC6 variants 6A & 6B

;01/24/10
;Removed user vars... turns out I don't need them here, and I didn't implement
;correctly anyways (caused CHR overdump with Esper Dream 2). Basically reverting
;back to version 2 of plugin.
             
             ;vectors for standard system calls

send_byte:   .equ 0200h
baton:       .equ 0203h
wr_ppu:      .equ 020ch
chk_wram:    .equ 0209h

temp1:       .equ 00e0h
temp1_lo:    .equ 00e0h
temp1_hi:    .equ 00e1h
temp2:       .equ 00e2h
temp2_lo:    .equ 00e2h
temp2_hi:    .equ 00e3h
temp3:       .equ 00e4h
temp3_lo:    .equ 00e4h
temp3_hi:    .equ 00e5h
temp4:       .equ 00e6h
temp4_lo:    .equ 00e6h
temp4_hi:    .equ 00e7h

temp_bank:   .equ 00ech
romsiz:      .equ 00edh
curr_bank:   .equ 00eeh


;plugin header defines
p_ver: = 4	;plugin version
p_read: = 1	;plugin is for downloading (dumping)
p_write: = 2	;plugin is for uploading (RAM carts / WRAM etc)
p_other: = 3	;plugin is for some other purpose (i.e. BankWatch)
p_var1: = 4	;plugin uses user variable #1
p_var2: = 8	;plugin uses user variable #2
p_var3: = 16	;plugin uses user variable #3
p_var4: = 32	;plugin uses user variable #4

;plugin header data

             .org 0380h
             
             ;plugin description
             .db "Konami VRC6 carts"
             .fill 03e0h-*,00h

             ;plugin creator(s)
             .db "Kevin Horton & BootGod"
             .fill 03f8h-*,00h

             ;PRG/CHR/WRAM min/max sizes supported
             .db 32 >> 4,256 >> 4
             .db 32 >> 3,256 >> 3
             .db 8,0

             ;plugin version and flags
             .db p_ver
             .db p_read

;plugin code start here

             lda #4
             jsr send_byte   ;send byte

;vrc version and variant must be defined, otherwise exit

             
             lda #0feh
             sta 0c000h      ;last 16K = last 16K on cart

             lda #1
             sta temp_bank
             lda #003h
             jsr test_bank
             beq got_it
             asl temp_bank
             lda #007h
             jsr test_bank
             beq got_it
             asl temp_bank


got_it:      ldx temp_bank
             lda #0
             jsr send_byte
             txa             ;send size
             jsr send_byte
             lda #001h
             jsr send_byte   ;send PRG 
             
             lda #0
             sta temp3_lo
             txa
             asl a
             asl a           ;4, 8 or 16 banks
             sta temp3_hi
            
             
di_loop2:    lda temp3_lo
             jsr wr_bank
             ldx #040h
             ldy #0
             sty temp1_lo
             lda #080h
             sta temp1_hi

dump_it:     lda (temp1),y
             jsr send_byte
             iny
             bne dump_it
             jsr baton
             inc temp1_hi
             dex
             bne dump_it
             inc temp3_lo
             dec temp3_hi
             bne di_loop2
                
;CHR             
             
no_ram3:     lda #1
             sta romsiz
             ldy #020h
             jsr comp_bank2
             beq gotit2
             asl romsiz
             ldy #040h
             jsr comp_bank2
             beq gotit2
             asl romsiz
             ldy #080h
             jsr comp_bank2
             beq gotit2
             asl romsiz
             
gotit2:      lda romsiz
             pha
             lsr romsiz
             lda #0
             sta temp3_hi    ;start out at 0
             ror a
             jsr send_byte
             lda romsiz
             jsr send_byte
             lda #02h
             jsr send_byte
             pla
             asl a
             asl a
             asl a
             asl a
             asl a  ;1,2,4,8 == 20,40,80,00
             sta temp3_lo

send_plp3:   lda temp3_hi
             sta 0d000h      ;current bank
             lda #00h
             jsr wr_ppu
             lda 2007h      ;set up PPU
             ldy #0
             ldx #4

send_plp4:   lda 2007h
             jsr send_byte
             iny
             bne send_plp4
             jsr baton
             dex
             bne send_plp4
             inc temp3_hi
             dec temp3_lo
             bne send_plp3
             
             lda #080h
             sta 0b003h
             jsr chk_wram
             bne done
             lda #020h
             tax
             jsr send_byte
             lda #0
             jsr send_byte
             lda #3
             jsr send_byte
             lda #0
             sta temp1_lo
             tay
             lda #060h
             sta temp1_hi
            
sr_lp:       lda (temp1),y
             jsr send_byte
             iny
             bne sr_lp
             inc temp1_hi
             jsr baton
             dex
             bne sr_lp
             lda #000h
             sta 0b003h
             

done:        lda #0
             jsr send_byte
             lda #0
             jsr send_byte
             lda #0
             jsr send_byte

             rts
             
wr_bank:     sta 08000h
             rts



test_bank:   jsr wr_bank
             lda #0
             sta temp1_lo
             sta temp2_lo
             lda #080h
             sta temp1_hi
             lda #0c0h
             sta temp2_hi
             
             ldx #040h
             ldy #0

chk_loopq:   lda (temp1),y
             cmp (temp2),y
             bne not_it
             iny
             bne chk_loopq
             inc temp1_hi
             inc temp2_hi
             dex
             bne chk_loopq

not_it:      rts



;y = bank to compare
;z=same
;for CHR
comp_bank2:  ldx #0
             stx temp3_lo    ;lower pointer
             sty temp3_hi    ;upper pointer
             sty temp4       ;# blocks to compare

cc_loop:     lda temp3_lo
             sta 0d000h
             lda temp3_hi
             sta 0e000h      ;write in current banks
             ldx #004h
             lda #000h
             sta curr_bank   ;reset current bank

cc_loop2:    ldy #0
             lda curr_bank
             sta 2006h       ;pointer =000h
             sty 2006h
             lda 2007h       ;garbage read

ql:          lda 2007h
             sta 0300h,y
             iny
             bne ql          ;load 256 bytes for testing
             lda curr_bank
             clc
             adc #010h
             inc curr_bank
             sta 2006h       ;pointer =1000h
             sty 2006h
             lda 2007h       ;garbage read

cc_loop3:    lda 2007h
             cmp 0300h,y
             bne diff2
             iny
             bne cc_loop3
             dex
             bne cc_loop2
             inc temp3_lo
             inc temp3_hi
             dec temp4
             bne cc_loop

diff2:       rts


             .fill 0800h-*,0ffh   ;fill rest to get 1K of data

             .end
             
